// PanelFlag class for Flag lab
// This class creates the panel for the flags

// (c) J. Dean Brock, Rebecca F. Bruce, 2001, 2004
// Department of Computer Science
// University of North Carolina at Asheville

package edu.unca.cs.csci201.LabAids;

/**
 * Creates panels for student's plots within the Flag lab
 */
public class PanelFlag extends edu.unca.cs.LabAids.PainterMux {
    
    private SpecFlag activeArtist ;

    /**
     * Contructor for PanelFlag
     * @param width Width of panel to <B>display</B> plots.
     * This is <B>not</B> the width of the frame.
     * 
     * @param height Height of panel to <B>display</B> plots.
     * This is <B>not</B> the height of the frame.
     */
   
    public PanelFlag(int width, int height) {
        super(width, height) ;
    }

    /**
     * Add a plotter
     * @param plotObject Object to the plotted.  Must follow the
     * SpecFlag interface.
     * @param plotName Name for button to select the plot
     */
    public void addPainter(SpecFlag plotObject, String plotName) {
        super.addPainter(new PainterFlag(plotObject), plotName) ;
    }
    
    private class PainterFlag implements edu.unca.cs.LabAids.Painter {
        SpecFlag painterFlag ;
        
        private class SyncStateData {
            public java.awt.Color[] progOutputs ;
            public boolean runFailure = false ;
            public boolean badDimension = false ;
            public Exception runException = null ;
            public boolean runComplete = false ;
            private int cellWidth = -1 ;
            private int cellHeight = -1 ;
            private int numCells = -1 ;
        } ;
        
        SyncStateData progState = new SyncStateData() ;
        
        public PainterFlag(SpecFlag NPflag) {
            painterFlag = NPflag ;
            
            Runnable computeColors = new Runnable() {
                public void run() {
                    try {
                        int cellWidth = painterFlag.getWidth() ;
                        int cellHeight = painterFlag.getHeight() ;
                        if (cellWidth <= 0 || cellHeight <= 0) {
                            synchronized(progState) {
                                progState.runFailure = true ;
                                progState.badDimension = true ;
                                progState.runComplete = true ;
                            }
                        } else {
                            int numCells ;
                            synchronized(progState) {
                                progState.cellWidth = cellWidth ;
                                progState.cellHeight = cellHeight ;
                                progState.numCells = numCells = cellWidth*cellHeight ;
                                progState.progOutputs = new java.awt.Color[numCells] ;
                            }
                            boolean goodAnswers = true ;
                            int xpos = 0 ;
                            int ypos = 0 ;
                            for (int i = 0; i < numCells && goodAnswers; ++i) {
                                java.awt.Color c = painterFlag.TestFunc(xpos,ypos) ;
                                synchronized(progState) {
                                    progState.progOutputs[i] = c ;
                                }
                                ++xpos ;
                                if (xpos == cellWidth) {
                                    xpos = 0 ;
                                    ++ypos ;
                                }
                            }
                        }
                    } catch (Exception e) {
                        synchronized(progState) {
                            progState.runFailure = true ;
                            progState.runException = e ;
                        }
                    }
                    synchronized(progState) {
                        progState.runComplete = true ;
                    }
                }
            } ;
            try {
                Thread computeThreads = new Thread(computeColors) ;
                // Let him run and hope he's done in time
                computeThreads.start() ;
            } catch (java.security.AccessControlException e) {
                // This hack allows the PainterMux to run under an applet
                // In that case, "infinite" loops with hang the applet
                computeColors.run() ;
            }
        }
        
        public void paintCanvas(java.awt.Graphics g, javax.swing.JComponent panel) {
            g = g.create() ;
            int myWidth = panel.getWidth() ;
            int myHeight = panel.getHeight() ;
            
            synchronized(progState) {
                if (!progState.runFailure && progState.runComplete) {
                    int halfWidth = progState.cellWidth/2 ;
                    int halfHeight = progState.cellHeight/2 ;
                    
                    int presX ;
                    int nextX = 0 ;
                    for (int i=0; i<progState.cellWidth; ++i) {
                        presX = nextX ;
                        nextX = ((i+1)*myWidth+halfWidth)/progState.cellWidth ;
                        int presY ;
                        int nextY = 0 ;
                        for (int j=0; j<progState.cellHeight; ++j) {
                            presY = nextY ;
                            nextY = ((j+1)*myHeight+halfHeight)/progState.cellHeight ;
                            int aPos = i + j*progState.cellWidth ;
                            java.awt.Color bColor = progState.progOutputs[aPos] ;
                            g.setColor( bColor ) ;
                            g.fillRect(presX, presY, nextX-presX, nextY-presY) ;
                        }
                    }
                } else {
                    String errMsg ;
                    if (!progState.runComplete)
                        errMsg = "Still running" ;
                    else if (progState.badDimension)
                        errMsg = "Bad size" ;
                    else 
                        errMsg = progState.runException.toString() ;
                    edu.unca.cs.LabAids.ErrorWriter.toComponent(errMsg, g, panel) ;
                }
            }
        }
    }
}

